//
//  G15Display.m
//  G15
//
//  Created by Casey Fleser on 9/12/07.
//  Copyright 2007 __MyCompanyName__. All rights reserved.
//

#import "G15Display.h"
#import "G15Manager.h"

static G15Display		*sDefaultDisplay = nil;

@implementation G15Display

+ (G15Display *) defaultDisplay
{
	return sDefaultDisplay;
}

- (void) awakeFromNib
{
	[[_mirrorText textContainer] setLineFragmentPadding: 0.0];
	[_mirrorText setDelegate: self];
}

- (id) init
{
	if ((self = [super init]) != nil) {
		NSBitmapImageRep	*imageRep;
		
		if (sDefaultDisplay != nil)
			[sDefaultDisplay release];
			
		sDefaultDisplay = self;
		memset(_lcdBuffer.control, 0x00, sizeof(_lcdBuffer.control));
		_lcdBuffer.control[0] = 0x03;

		
		_displayImage = [[NSImage alloc] initWithSize: NSMakeSize(160, 43)];
		[_displayImage setFlipped: YES];
		imageRep = [[NSBitmapImageRep alloc] initWithBitmapDataPlanes: nil
												pixelsWide: 160
												pixelsHigh: 43
												bitsPerSample: 1
												samplesPerPixel: 1
												hasAlpha: NO
												isPlanar: YES
												colorSpaceName: NSCalibratedBlackColorSpace
												bytesPerRow: 0
												bitsPerPixel: 0];

		[_displayImage addRepresentation: imageRep];

	}
	
	return self;
}

- (void) drawViewAndSubs: (NSView *) inView
{
	NSArray				*subViews = [inView subviews];
	NSScrollView		*enclosingScroller = [inView enclosingScrollView];
	
	if (enclosingScroller != nil) {
		NSAffineTransform	*transform = [NSAffineTransform transform];
		NSRect				visRect = [enclosingScroller documentVisibleRect];
		
		[NSGraphicsContext saveGraphicsState];
		[transform translateXBy: visRect.origin.x yBy: -visRect.origin.y];
		[transform concat];
	}

	[inView drawRect: [inView bounds]];
	
	if (enclosingScroller != nil) {
		[NSGraphicsContext restoreGraphicsState];
	}
	
	if (subViews != nil && [subViews count]) {
		NSEnumerator		*viewEnumerator = [subViews objectEnumerator];
		NSView				*view;
		
		while ((view = [viewEnumerator nextObject]) != nil) {
			if (![view isHidden]) {
				NSAffineTransform	*transform = [NSAffineTransform transform];
				
				[NSGraphicsContext saveGraphicsState];
				[transform translateXBy: [view frame].origin.x yBy: [view frame].origin.y];
				[transform concat];

				[[NSBezierPath bezierPathWithRect: NSOffsetRect([inView frame], -NSMinX([inView frame]), -NSMinY([inView frame]))] addClip];
				
				[self drawViewAndSubs: view];
				[NSGraphicsContext restoreGraphicsState];
			}
		}
	}
}

- (void) updateDisplay
{
	NSBitmapImageRep	*rep;
	NSRect				visRect;
	G15Device			*device;
	int					x, y;
	unsigned int		pixData[4];
	float				lum;
	
	visRect = [[_mirrorText enclosingScrollView] documentVisibleRect];
	[_displayImage lockFocus];
	
	[self drawViewAndSubs: [_mirrorText enclosingScrollView]];

	rep = [[NSBitmapImageRep alloc] initWithFocusedViewRect: NSMakeRect(0, 0, 160, 43)];
	[_displayImage unlockFocus];

	memset(_lcdBuffer.buffer, 0x00, sizeof(_lcdBuffer.buffer));

	for (x=0; x<160; x++) {
		for (y=0; y<43; y++) {
			[rep getPixel: pixData atX: x y: y];
			lum = ((float)pixData[0] / 255.0) * 0.30 + ((float)pixData[1] / 255.0) * 0.59 + ((float)pixData[2] / 255.0) * 0.11;
			if (lum < 0.60) {
				_lcdBuffer.buffer[((y / 0x08) * 0xa0) + x] |= 1 << (y % 8);
			}
		}
	}
	
	device = [[G15Manager defaultManager] deviceAtIndex: 0];
	if (device != nil)
		[device sendLCDData: &_lcdBuffer];

	[rep release];
}

- (void) performClear
{
	[[_mirrorText textStorage] deleteCharactersInRange: NSMakeRange(0, [[_mirrorText textStorage] length])];
	[self updateDisplay];
}

- (void) performHome
{
	[_mirrorText scrollRangeToVisible: NSMakeRange(0, 0)];
	[self updateDisplay];
}

- (void) performScrollUp
{
	[_mirrorText scrollPageUp: nil];
	[self updateDisplay];
}

- (void) performScrollDown
{
	[_mirrorText scrollPageDown: nil];
	[self updateDisplay];
}

- (void) performEnd
{
	[_mirrorText scrollRangeToVisible: NSMakeRange([[_mirrorText textStorage] length] - 1, 1)];
	[self updateDisplay];
}

- (void) appendAttributedString: (NSAttributedString *) inAttributedString
{
	[[_mirrorText textStorage] appendAttributedString: inAttributedString];
	[_mirrorText scrollRangeToVisible: NSMakeRange([[_mirrorText textStorage] length] - 1, 1)];

	[self updateDisplay];
}

- (void) textDidChange: (NSNotification *) inNotification
{
	NSLog(@"string %@", [_mirrorText string]);
	[self updateDisplay];
}

@end
