//
//  G15Manager.m
//  G15
//
//  Created by Casey Fleser on 9/12/07.
//  Copyright 2007 __MyCompanyName__. All rights reserved.
//

#import "G15Manager.h"
#import "G15Device.h"
#import <IOKit/IOKitLib.h>
#import <IOKit/hid/IOHIDLib.h>
#import <IOKit/usb/IOUSBLib.h>

static G15Manager		*sG15Mgr = nil;

static void DeviceAdded(
	void			*inRefCon,
	io_iterator_t   inIterator)
{
	G15Manager		*monitor = (G15Manager *)inRefCon;
	G15Device		*device;
	io_service_t	obj;

	while (obj = IOIteratorNext(inIterator)) {
		if ((device = [[G15Device alloc] initWithService: obj]) == nil)
			IOObjectRelease(obj);
		else {
			[monitor addDevice: device];
			[device release];
		}
	}
}

static void DeviceRemoved(
	void			*inRefCon,
	io_iterator_t   inIterator)
{
	G15Manager	*monitor = (G15Manager *)inRefCon;
	io_service_t	obj;

	while (obj = IOIteratorNext(inIterator)) {
		[monitor removeDevice: obj];
	}
}

@implementation G15Manager

+ (G15Manager *) defaultManager
{
	if (sG15Mgr == nil)
		sG15Mgr = [[G15Manager alloc] init];
	
	return sG15Mgr;
}

- (void) awakeFromNib
{
	G15Manager		*g15Manager = [G15Manager defaultManager];
	
	[g15Manager start];
}

- (id) init
{
	if ((self = [super init]) != nil)
		_devices = [[NSMutableArray array] retain];
	
	return self;
}

- (IOReturn) start
{
	IOReturn				result;
	CFMutableDictionaryRef  matchingDict;
	
	if ((matchingDict = IOServiceMatching(kIOHIDDeviceKey)) == nil) {
		NSLog(@"IOServiceMatching failed File %s Line %d", __FILE__, __LINE__);
		result = kIOReturnError;
	}
	else {
		NSMutableDictionary		*dict = (NSMutableDictionary *)matchingDict;
		
		CFRetain(matchingDict);
		[dict setValue: [NSNumber numberWithLong: kG15Product] forKey: [NSString stringWithCString: kIOHIDProductIDKey]];
		[dict setValue: [NSNumber numberWithLong: kG15Vendor] forKey: [NSString stringWithCString: kIOHIDVendorIDKey]];
		
		_notifyPort = IONotificationPortCreate(kIOMasterPortDefault);
		result = IOServiceAddMatchingNotification(_notifyPort, kIOFirstMatchNotification, matchingDict, &DeviceAdded, self, &_deviceIterator);
		
		if (result != KERN_SUCCESS) {
			NSLog(@"IOServiceAddMatchingNotification failed File %s Line %d - result %08x", __FILE__, __LINE__, result);
		}
		else {
			DeviceAdded((void *)self, _deviceIterator);
			
			result = IOServiceAddMatchingNotification(_notifyPort, kIOTerminatedNotification, matchingDict, &DeviceRemoved, self, &_deviceIterator);
			
			if (result != KERN_SUCCESS) {
				NSLog(@"IOServiceAddMatchingNotification failed File %s Line %d - result %08x", __FILE__, __LINE__, result);
			}
			else {
				CFRunLoopSourceRef 		runLoopSource;
				
				DeviceRemoved((void *)self, _deviceIterator);

				runLoopSource = IONotificationPortGetRunLoopSource(_notifyPort);
				CFRunLoopAddSource(CFRunLoopGetCurrent(), runLoopSource, kCFRunLoopDefaultMode);
				CFRelease(runLoopSource);
			}
		}
		
	}
	
	return result;
}

- (void) addDevice: (G15Device *) inDevice
{
	[_devices addObject: inDevice];
}

- (void) removeDevice: (io_service_t) inServiceID
{
	NSEnumerator	*deviceEnumerator = [_devices objectEnumerator];
	G15Device		*device;
	
	while ((device = [deviceEnumerator nextObject]) != nil) {
		if ([G15Device locationID: [device serviceID]] == [G15Device locationID: inServiceID]) {
			[device shutdownDevice];
			[_devices removeObject: device];
			break;
		}
	}
}

- (NSArray *) devices
{
	return _devices;
}

- (G15Device *) deviceAtIndex: (unsigned) inIndex
{
	G15Device		*device = nil;
	
	if (inIndex < [_devices count])
		device = [_devices objectAtIndex: inIndex];
		
	return device;
}

@end
